package rabinizer.exec;

/* 
 * @ author Jan Kretinsky & Zuzana Komarkova 
 * */ 

import java.util.*;
import java.util.Map.Entry;

import rabinizer.formulas.*;

public class KDFA {

	/*
      second_component := List<Set<Formula>>, List<Set<Formula>>
      state := Tuple<Formula, second_component> := Tuple<Formula, List<Set<Formula>>>
	 */
	public Set<Tuple<Formula, KState>> states = null;
	public Tuple<Formula, KState> initialState = null;
	public Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edges = null;
	public Map<Formula, Formula> displayLabels = null;
	public List<SegmentAutomaton> segmentAutomata = null;
	public Formula phi = null;
	public Map<Formula, Integer> formulaToSegmentId = null;
	public Map<Formula,Integer> sAToMaxSinkRank = null;
	//statesToNumbers - as generated, starting from 0
	public Map<Tuple<Formula,KState>,Integer> statesToNumbers=null;
	public Integer currentNumber;
	public String str;
	/*
    // GeneralizedRabinAcceptanceCondition.class uses the following three
    public List<SegmentAutomaton> segmentAutomata = null;
    public Formula phi = null;
    public Map<Formula, Integer> rFtoSAid = null;
	 */
	/**
	 * Algorithm DFA
	 * 
	 * @param phi
	 */
	public KDFA(Formula phi, List<SegmentAutomaton> sAs, KState iSSs, List<Valuation> valuations) {

		Misc.verboseln("DFA for " + phi);
		
		
		for (SegmentAutomaton sa : sAs)
			if(phi.unfold().representative().hasSuspendableG(new GOperator(sa.displayLabels.get(sa.initialState)))) 
				iSSs.removeSA(sa);
		initialState = new Tuple<Formula, KState>(phi.unfold().representative(), iSSs);

		states = new HashSet<Tuple<Formula, KState>>();
		statesToNumbers= new HashMap<Tuple<Formula, KState>,Integer>();
		currentNumber=0;
		segmentAutomata = sAs;
		Map<Formula, Integer> formulaToSegmentId  = new HashMap<Formula, Integer>(segmentAutomata.size());
		for (int i = 0; i < segmentAutomata.size(); i++)
			formulaToSegmentId.put(segmentAutomata.get(i).initialState, i);
		sAToMaxSinkRank = new HashMap<Formula,Integer>(segmentAutomata.size());
		for(SegmentAutomaton sa:segmentAutomata)
			sAToMaxSinkRank.put(sa.initialState, 0);
		states.add(initialState);
		statesToNumbers.put(initialState,currentNumber);
		currentNumber++;
		this.phi=phi;
		edges = new HashMap<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>();
		Stack<Tuple<Formula, KState>> workstack = new Stack<Tuple<Formula, KState>>();
		displayLabels = new HashMap<Formula, Formula>();
		displayLabels.put(phi.unfold().representative(), phi);
		workstack.push(initialState);

		int loopCount = 0;
		while (!workstack.empty()) {

			Misc.verboseln("loop count: " + loopCount++);
			Misc.verboseln("workstack : " + workstack);
			Misc.verboseln("workcount : " + workstack.size());

			Tuple<Formula, KState> curr = workstack.pop();
			Misc.verboseln("Curr: " + curr);

			for (Valuation v : valuations) {
				Misc.verboseln("\tv: " + v);

				// successor state
				Formula displayLabel = curr.left.temporalStep(v);
				Formula stateLabel = displayLabel.unfold().representative();
				
				
				Set<SegmentAutomaton> relevantSAs = new HashSet<SegmentAutomaton>();
				for(SegmentAutomaton sa : sAs)
					if(stateLabel.hasSubformula(sa.displayLabels.get(sa.initialState)))         // no more relevant
						if(! stateLabel.hasSuspendableG(new GOperator(sa.displayLabels.get(sa.initialState)))) // not yet relevant
							relevantSAs.add(sa);
				KState Ksucc = curr.right.step(relevantSAs, v);
				Tuple<Formula, KState> succ = new Tuple<Formula, KState>(stateLabel, Ksucc);
				Misc.verboseln("\tSucc: " + succ);

				Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>> currAndSucc = new Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>(curr, succ);
				if (edges.containsKey(currAndSucc)) {
					Misc.verboseln("\tUpdate edge");
					ValuationSet label = edges.get(currAndSucc);
					label.add(v);
					edges.put(currAndSucc, label);
				} else {
					Misc.verboseln("\tNew edge");
					ValuationSet label = new ValuationSet();
					label.add(v);
					edges.put(currAndSucc, label);
				}
				if (!states.contains(succ)) {
					Misc.verboseln("\tNew state");
					states.add(succ);
					statesToNumbers.put(succ,currentNumber);
					currentNumber++;
					workstack.push(succ);
				}
				if (!displayLabels.containsKey(stateLabel)) {
					Misc.verboseln("\tNew display label: " + displayLabel);
					displayLabels.put(stateLabel, displayLabel);
				}
			}
		}
		countMax();
		Misc.verboseln("# states     : " + states.size());
		Misc.verboseln("# edges      : " + edges.size());
		Misc.verboseln("Initial state: " + initialState);
		Misc.verboseln("States       : " + states);
		Misc.verboseln("States mapping       : " + statesToNumbers);
		Misc.verboseln("Edges        : " + edges);


	}
	/* public void countMax(){
    	for (Map.Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : edges.entrySet()) {
			Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>> edgeKey = edge.getKey();
			KState kstate1 = edgeKey.left.right;
			for(Valuation v:edge.getValue()){
				for(SegmentAutomaton sa: kstate1.state.keySet()){
					for(Formula f:kstate1.state.get(sa).keySet()){
						if(sa.finalStates.contains(sa.tranFunction.get(f).get(v))){
							int currInt = kstate1.state.get(sa).get(f);
		        			if(currInt > sAToMaxSinkRank.get(sa.initialState))
		        				sAToMaxSinkRank.put(sa.initialState, currInt);
						}
					}

				}
			}

    	}

    }
	 */

	public void countMax(){
		for (Map.Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : edges.entrySet()) {
			Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>> edgeKey = edge.getKey();
			KState kstate1 = edgeKey.left.right;
			for(Valuation v:edge.getValue()){
				for(SegmentAutomaton sa: kstate1.state.keySet()){
					for(Formula f:kstate1.state.get(sa).keySet()){
						//if(sa.finalStates.contains(sa.tranFunction.get(f).get(v))){
						int currInt = kstate1.state.get(sa).get(f);
						if(currInt > sAToMaxSinkRank.get(sa.initialState))
							sAToMaxSinkRank.put(sa.initialState, currInt);
					}

				}
			}

		}

	}



	public String toDotty() {
		String dot = "digraph \"GenRabinAutomaton " + initialState +"\" {\n";
		//        Misc.verboseln("States       : " + states);
		//        Misc.verboseln("# states     : " + states.size());
		// The initial state in the set states may be different from the states in the set of edges. This can happen
		// because HashSet does not guarantee that two segment states that are equal have the same string representation.
		// To circumvent the problem, use the same string representation for rendering nodes and edges in the dotty file.
		Map<Tuple<Formula, KState>, Tuple<Formula, KState>> normalStates = new HashMap<Tuple<Formula, KState>, Tuple<Formula, KState>>();        
		for (Tuple<Formula, KState> s : states) {
			normalStates.put(s,s);
			Misc.verbose(",");
			Formula display = displayLabels.get(s.left);

			KState k = s.right;
			String display2 = "";
			for(SegmentAutomaton sa : k.state.keySet()){
				//List<Formula> fs = (List<Formula>) k.state.get(sa).keySet();
				display2 += "[";
				for(Formula f : k.state.get(sa).keySet())
					display2 += "" + sa.displayLabels.get(f) + ":" + k.state.get(sa).get(f) + " ";
				display2 += "] ";
			}
			if (s.equals(initialState)) {
				dot += "\nnode [shape=oval, label=\"" + display + " :: " + display2 + "\"]\"" + s + "\";\n";
			} else {
				dot += "\nnode [shape=rectangle, label=\"" + display + " :: " + display2 + "\"]\"" + s + "\";\n";
			}
		}
		for (Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : edges.entrySet()) {
			Misc.verbose(".");
			dot += "\n\"" + normalStates.get(edge.getKey().left) + "\" -> \"" + normalStates.get(edge.getKey().right) + "\" [label=\"" + BDDService.BDDtoString(BDDService.valuationSetToBDD(edge.getValue())) + "\"];\n"; 
		}
		return dot + "}\n";
	}
	
	public String toDottyNotBDD() {
		String dot = "digraph \"GenRabinAutomaton " + initialState +"\" {\n";
		
		Map<Tuple<Formula, KState>, Tuple<Formula, KState>> normalStates = new HashMap<Tuple<Formula, KState>, Tuple<Formula, KState>>();        
		for (Tuple<Formula, KState> s : states) {
			normalStates.put(s,s);
			Misc.verbose(",");
			Formula display = displayLabels.get(s.left);

			KState k = s.right;
			String display2 = "";
			for(SegmentAutomaton sa : k.state.keySet()){
				//List<Formula> fs = (List<Formula>) k.state.get(sa).keySet();
				display2 += "[";
				for(Formula f : k.state.get(sa).keySet())
					display2 += "" + sa.displayLabels.get(f) + ":" + k.state.get(sa).get(f) + " ";
				display2 += "] ";
			}
			if (s.equals(initialState)) {
				dot += "\nnode [shape=oval, label=\"" + display + " :: " + display2 + "\"]\"" + s + "\";\n";
			} else {
				dot += "\nnode [shape=rectangle, label=\"" + display + " :: " + display2 + "\"]\"" + s + "\";\n";
			}
		}
		for (Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : edges.entrySet()) {
			Misc.verbose(".");
			dot += "\n\"" + normalStates.get(edge.getKey().left) + "\" -> \"" + normalStates.get(edge.getKey().right) + "\" [label=\"" + edge.getValue() + "\"];\n"; 
		}
		return dot + "}\n";
	}

	public String toDottyNumbered() {
		String dot = "digraph \"GenRabinAutomaton " + initialState +"\" {\n";
		
		Map<Tuple<Formula, KState>, Tuple<Formula, KState>> normalStates = new HashMap<Tuple<Formula, KState>, Tuple<Formula, KState>>();        
		for (Tuple<Formula, KState> s : states) {
			normalStates.put(s,s);
			Misc.verbose(",");
			Formula display = displayLabels.get(s.left);

			
			if (s.equals(initialState)) {
				dot += "\nnode [shape=oval, label=\""+statesToNumbers.get(s)+ " :: " + display + "\"]\"" + s + "\";\n";
			} else {
				dot += "\nnode [shape=rectangle, label=\""+statesToNumbers.get(s)+ " :: "  + display  + "\"]\"" + s + "\";\n";
			}
		}
		for (Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : edges.entrySet()) {
			Misc.verbose(".");
			dot += "\n\"" + normalStates.get(edge.getKey().left) + "\" -> \"" + normalStates.get(edge.getKey().right) + "\" [label=\"" + BDDService.BDDtoString(BDDService.valuationSetToBDD(edge.getValue())) + "\"];\n"; 
		}
		return dot + "}\n";
	}

	public String toNumerical(GeneralizedRabinAcceptanceCondition condition) {
		String dot = "States: " + states.size() +"\n";
		dot +="Start: " + statesToNumbers.get(initialState) +"\n";
		dot +="Acceptance: " + condition.toNumericalString() +"\n";
		dot+="AP: "+Misc.bijectionIdAtom.size();
		for(Integer i=0;i<Misc.bijectionIdAtom.size();i++){
			if(i==Misc.bijectionIdAtom.size()-1){dot+=" \"" + Misc.bijectionIdAtom.atom(i)+"\" \n";}
			else dot+=" \"" + Misc.bijectionIdAtom.atom(i)+"\"";
		}

		// Map<Tuple<Formula, KState>, Tuple<Formula, KState>> normalStates = new HashMap<Tuple<Formula, KState>, Tuple<Formula, KState>>();        
		for (Tuple<Formula, KState> s:states) {
			dot+="State: "+ statesToNumbers.get(s)+" \""+ s + "\" \n";
			List<String> atoms= new ArrayList<String>();
			for(Integer i=Misc.bijectionIdAtom.size()-1;i>=0;i--){
				atoms.add((Misc.bijectionIdAtom.size()-1-i), Misc.bijectionIdAtom.atom(i));
			}
			str="";
			iterateValuations(atoms,0,atoms.size(), s, condition);

			dot+=str;
			dot+="\n";
		}
		return dot + "\n";
	}
	
	public String toPrism(int NrOfAp) {
		String a = "" + states.size() + " states "
				   + "(start " + statesToNumbers.get(initialState) + "), " 
				   + NrOfAp + " labels:\n";
		for (Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : edges.entrySet()) {
			for (Valuation v : edge.getValue()) {
				a += " " + statesToNumbers.get(edge.getKey().left) + "-";
				a += v.toString();
				a += "->" + statesToNumbers.get(edge.getKey().right) + "\n";
			}
		}
		return a + "; ";
	}
	
	private void iterateValuations(List<String> atoms, int pos, int count, Tuple<Formula, KState> s,GeneralizedRabinAcceptanceCondition condition) {
		if (count <= 0) {
			Valuation val= new Valuation();
			for(String a:atoms)
				val.put(Misc.bijectionIdAtom.id(a),true);

			for(Integer i : Misc.bijectionIdAtom.idToAtom.keySet()){
				if(!val.containsKey(i)) val.put(i,false);
			}
			for(Map.Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : edges.entrySet()){
				if(edge.getKey().left.equals(s)){
					if(edge.getValue().contains(val)){
						str+=""+statesToNumbers.get(edge.getKey().right)+" ";
						int count2=0;
						for(Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edgeC: condition.edgesToInt.keySet()){
							Misc.verboseln("hrany"+edgeC+"hrana, co zkousim" + edge);
							if(edgeC.keySet().contains(edge.getKey())&&edgeC.get(edge.getKey()).contains(val)){
								count2++;
								if (count2==1)str+="{";
								if (count2>1)str+=" ";
								str+=condition.edgesToInt.get(edgeC);
							}
						}
						if(count2>0){str+="}";}
						str+="\n";
					}
				}
			}

		} else {
			// remove element in position pos 
			List<String> currPosRemoved = new ArrayList<String>(atoms.size());
			currPosRemoved.addAll(atoms);
			currPosRemoved.remove(pos);
			iterateValuations(currPosRemoved, pos, count - 1, s,condition);
			// keep element in position pos
			iterateValuations(atoms, pos + 1, count - 1, s,condition);



		}
	}


}
