package rabinizer.formulas;


import java.util.*;

import rabinizer.exec.*;
import net.sf.javabdd.*;

/**
 * @author Andreas Gaiser & Ruslan Ledesma-Garza
 *
 */
public abstract class Formula {

	String strFormula = null;

	BDD cachedBdd = null;

	public abstract BDD bdd();

	public Formula representative() {
		return Misc.representativeOfBdd(bdd(), this);
	}

	// TODO: in each subclass, revise hashCode()
	public abstract int hashCode();

	/**
	 * Checks structural equality.
	 */
	public abstract boolean equals(Object o);

	/* TODO: deprecated */
	public abstract Formula negated();

	public abstract String toReversePolishString();

	/**
	 * Algorithm R
	 * @return
	 */
	public Tuple<Set<Formula>, Set<Formula>>  recurrentFormulas() { 
		return this.recurrentFormulas(false);
	}

	/**
	 * Algorithm R
	 * @return
	 */	
	protected abstract Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate);

	/**
	 * Subprocedure .[.]
	 * @param valuation
	 * @return
	 */
	public abstract Formula evaluateCurrentAssertions(Valuation valuation);

	/**
	 * Subprocedure Remove
	 * @param valuation
	 * @return
	 */
	public abstract Formula removeConstantCurrentAssertions();

	/**
	 * Subprocedure Assert
	 * @return
	 */
	public Formula assertValuation(Valuation valuation) {
		return evaluateCurrentAssertions(valuation).removeConstantCurrentAssertions();
	}

	/**
	 * Subprocedure RemoveX
	 * @return
	 */
	public abstract Formula removeX();

	/**
	 * Subprocedure TemporalStep
	 * @param valuation
	 * @return
	 */
	public Formula temporalStep(Valuation valuation) {
		return this.assertValuation(valuation).removeX();
	}

	/**
	 * Subprocedure Unfold
	 * @return
	 */
	public abstract Formula unfold();

	// unfold everything but G's, used in segment automata
	public abstract Formula unfoldNoG();

	/**
	 * TODO: include in documentation
	 * @return
	 */
	public abstract boolean isProgressFormula();

	public boolean hasSuspendableG(Formula f) {
		return false;
	}
	
	public boolean containsG() {
		return false;
	}
	
	public boolean isVeryDifferentFrom(Formula f) {
		return false;
	}
	
	public boolean isUnfoldOfF() {
		return false;
	}

	public abstract boolean hasSubformula(Formula f);

	public abstract Formula removeXsFromCurrentBooleanAtoms();

	public abstract Set<Formula> argumentsAlways();

	public abstract Set<Formula> argumentsFinsideG(boolean acc);

	public abstract boolean untilOcurrs();

	public boolean untilInsideForall() {
		for (Formula f : this.argumentsAlways())
			if (f.untilOcurrs()) return true;
		return false;
	}

	public abstract Formula toNNF();
}
