package tests;

import static org.junit.Assert.*;

import java.io.StringReader;
import java.util.Arrays;
import java.util.Collection;

import org.junit.Test;

import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import rabinizer.formulas.Formula;
import rabinizer.parser.LTLParser;

@RunWith(value = Parameterized.class)
public class RemoveXsFromCurrentBooleanAtoms {

	private String input;
	private String output;
	private boolean expected;
	
	public RemoveXsFromCurrentBooleanAtoms(String input, String output, boolean expected) {
		this.input = input;
		this.output = output;
		this.expected = expected;
	}

    @Parameters
	public static Collection<Object[]> data() {
        Object[][] data = new Object[][] { 
        		{ "X X G a & X a", "G a & X a", true },
        		{ "X X F a & X a", "F a & X a", true },
        		{ "X (X G a & a)", "X (X G a & a)", true },
        		{ "X F b", "F b", true },
        		
        		{ "X (X G a & a)", "X G a & a", false },
        };
        return Arrays.asList(data);
    }

	@Test
	public void pushTest() {
		System.out.println(this.getClass().getCanonicalName());
		
		LTLParser parser = new LTLParser(new StringReader(input));
		Formula inf = null;
		try { inf = parser.parse(); }
		catch (Exception e) { assertTrue(false); }
		
		parser.ReInit(new StringReader(output));
		Formula outf = null;
		try { outf = parser.parsePreviousAtoms(); }
		catch (Exception e) { assertTrue(false); }
		
		System.out.println("input   :"+inf);
		System.out.println("output  :"+outf);
		System.out.println("expected:"+expected);
		assertTrue(inf.removeXsFromCurrentBooleanAtoms().equals(outf) == expected);
	}
}
